import type { AggregateDatabase } from "../interfaces.js";
// import { stockDatabase as stockDatabaseClickhouse } from "../stockdb.clickhouse.js";
// import { stockDatabase as stockDatabaseLmdbx } from "../stockdb.lmdbx.js";
import { optionContractDatabase as optionContractDatabaseClickhouse } from "../optiondb.clickhouse.js";
import { optionContractDatabase as optionContractDatabaseLmdbx } from "../optiondb.lmdbx.js";
import { nextDate } from "../lib/util.js";

async function syncAggregates<T>({
  fromDatabase,
  toDatabase,
  key,
  date,
}: {
  fromDatabase: AggregateDatabase<T>;
  toDatabase: AggregateDatabase<T>;
  key: T;
  date: string;
}) {
  const maxRetries = 3; // Maximum number of retries
  let retryCount = 0;
  
  while (retryCount < maxRetries) {
    try {
      const aggregatesFrom = (await fromDatabase.getAggregates({ key, date })).map(
        (aggregateWithoutKey) => ({ ...aggregateWithoutKey, key })
      );
      await toDatabase.insertAggregates(aggregatesFrom);
      break; // Exit the loop if successful
    } catch (error) {
      retryCount++;
      const delay = Math.pow(2, retryCount) * 1000; // Exponential backoff strategy
      console.warn(`Retrying due to error: ${error}. Retry count: ${retryCount}, Delay: ${delay}ms`);
      await new Promise((resolve) => setTimeout(resolve, delay));
    }
  }
  
  if (retryCount === maxRetries) {
    console.error(`Failed to sync aggregates after ${maxRetries} retries.`);
  }
}

const symbols = ["AMD", "AAPL", "MSFT", "GOOGL", "NFLX", "NVDA"];
async function run() {
  const startDate = process.argv[2];
  const endDate = process.argv[3];

  if (!startDate || !endDate) {
    console.error("Usage: node clickhouse-to-lmdbx.js <startDate> <endDate>");
    console.error("Dates should be in YYYY-MM-DD format");
    process.exit(1);
  }

  while (retryCount < maxRetries) {
    try {
      // ... rest of the code remains unchanged
      
      await run();
    } catch (error) {
      console.warn(`Error occurred: ${error}. Retrying...`);
      retryCount++;
      const delay = Mathe.pow(2, retryCount) * 1000; // Exponential backoff strategy
      console.warn(`Retry count: ${retryCount}, Delay: ${delay}ms`);
      await new Promise((resolve) => setTimeout(resolve, delay));
    }
  }
  
  if (retryCount === maxRetries) {
    console.error(`Failed to sync aggregates after ${maxRetries} retries.`);
  }
}
